######################################################################################################################
# Copyright (C) 2017-2022 Spine project consortium
# This file is part of Spine Items.
# Spine Items is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
# Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
# any later version. This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General
# Public License for more details. You should have received a copy of the GNU Lesser General Public License along with
# this program. If not, see <http://www.gnu.org/licenses/>.
######################################################################################################################

"""
Undo/redo commands for the Tool project item.

"""
import copy
from spine_items.commands import SpineToolboxCommand


class UpdateToolExecuteInWorkCommand(SpineToolboxCommand):
    def __init__(self, tool, execute_in_work):
        """Command to update Tool execute_in_work setting.

        Args:
            tool (Tool): the Tool
            execute_in_work (bool): True or False
        """
        super().__init__()
        self.tool = tool
        self.execute_in_work = execute_in_work
        self.setText(f"change execute in work setting of {tool.name}")

    def redo(self):
        self.tool.do_update_execution_mode(self.execute_in_work)

    def undo(self):
        self.tool.do_update_execution_mode(not self.execute_in_work)


class UpdateToolOptionsCommand(SpineToolboxCommand):
    def __init__(self, tool, options):
        """Command to update Tool options.

        Args:
            tool (Tool): the Tool
            options (dict): The options that change
        """
        super().__init__()
        self.tool = tool
        self.old_options = copy.deepcopy(self.tool._options)
        self.new_options = copy.deepcopy(self.tool._options)
        self.new_options.update(options)
        self.setText(f"change options of {tool.name}")

    def redo(self):
        self.tool.do_set_options(self.new_options)
        self.setObsolete(self.old_options == self.new_options)

    def undo(self):
        self.tool.do_set_options(self.old_options)


class UpdateKillCompletedProcesses(SpineToolboxCommand):
    def __init__(self, tool, kill_completed_processes):
        """Command to update Tool kill_completed_processes flag.

        Args:
            tool (Tool): the Tool
            kill_completed_processes (bool): new flag value
        """
        super().__init__()
        self._tool = tool
        self._kill_completed_processes = kill_completed_processes
        self.setText(f"change kill consoles setting of {tool.name}")

    def redo(self):
        self._tool.do_update_kill_completed_processes(self._kill_completed_processes)

    def undo(self):
        self._tool.do_update_kill_completed_processes(not self._kill_completed_processes)


class UpdateLogProcessOutput(SpineToolboxCommand):
    def __init__(self, tool, log_process_output):
        """Command to update Tool log_process_output flag.

        Args:
            tool (Tool): the Tool
            log_process_output (bool): new flag value
        """
        super().__init__()
        self._tool = tool
        self._log_process_output = log_process_output
        self.setText(f"change log process output setting of {tool.name}")

    def redo(self):
        self._tool.do_update_log_process_output(self._log_process_output)

    def undo(self):
        self._tool.do_update_log_process_output(not self._log_process_output)



#!/usr/bin/env python

from pathlib import Path
import time


current_year = time.gmtime().tm_year
root_dir = Path(__file__).parent.parent
project_source_dir = Path(root_dir, "spine_items")
test_source_dir = Path(root_dir, "tests")

expected = f"# Copyright (C) 2017-{current_year} Spine project consortium"


def update_copyrights(path, suffix, recursive=True):
    for path in path.iterdir():
        if path.suffix == suffix:
            i = 0
            with open(path) as python_file:
                lines = python_file.readlines()
                for i, line in enumerate(lines[1:4]):
                    if line.startswith("# Copyright (C) "):
                        lines[i + 1] = lines[i + 1][:21] + str(current_year) + lines[i + 1][25:]
                        break
            if len(lines) <= i + 1 or not lines[i + 1].startswith(expected):
                print(f"Confusing or no copyright: {path}")
            else:
                with open(path, "w") as python_file:
                    python_file.writelines(lines)
        elif recursive and path.is_dir():
            update_copyrights(path, suffix)


update_copyrights(root_dir, ".py", recursive=False)
update_copyrights(project_source_dir, ".py")
update_copyrights(project_source_dir, ".ui")
update_copyrights(test_source_dir, ".py")

print("Done. Don't forget to update append_license.py!")



######################################################################################################################
# Copyright (C) 2017-2022 Spine project consortium
# This file is part of Spine Items.
# Spine Items is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
# Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
# any later version. This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General
# Public License for more details. You should have received a copy of the GNU Lesser General Public License along with
# this program. If not, see <http://www.gnu.org/licenses/>.
######################################################################################################################
"""Unit tests for ``tool_specifications`` module."""
import unittest
from unittest.mock import MagicMock
from spine_items.tool.tool_specifications import ToolSpecification


class TestToolSpecification(unittest.TestCase):
    def test_to_dict_sorts_input_and_output_files(self):
        app_settings = MagicMock()
        logger = MagicMock()
        specification = ToolSpecification(
            "specification name",
            "Python",
            "/path/to/tool",
            ["/included_file_1.bat", "/aa/included_file_2.std"],
            app_settings,
            logger,
            "Test tool specification",
            ["b_input.png", "a_input.jpg"],
            ["*.dat", "a.csv", "z.zip", "*.xlsx"],
            ["*.zip", "*.atk"],
            ["99", "10"],
        )
        specification.definition_file_path = "/path/to/specification/file.json"
        specification_dict = specification.to_dict()
        self.assertEqual(
            specification_dict,
            {
                "name": "specification name",
                "tooltype": "Python",
                "includes": ["/included_file_1.bat", "/aa/included_file_2.std"],
                "description": "Test tool specification",
                "inputfiles": ["a_input.jpg", "b_input.png"],
                "inputfiles_opt": ["*.dat", "*.xlsx", "a.csv", "z.zip"],
                "outputfiles": ["*.atk", "*.zip"],
                "cmdline_args": ["99", "10"],
                "includes_main_path": "../tool",
            },
        )


if __name__ == '__main__':
    unittest.main()



######################################################################################################################
# Copyright (C) 2017-2022 Spine project consortium
# This file is part of Spine Items.
# Spine Items is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
# Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
# any later version. This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General
# Public License for more details. You should have received a copy of the GNU Lesser General Public License along with
# this program. If not, see <http://www.gnu.org/licenses/>.
######################################################################################################################

"""
Unit tests for PythonToolSpecOptionalWidget class.

"""

import unittest
from unittest import mock
from unittest.mock import MagicMock
import logging
import sys
from PySide6.QtWidgets import QApplication, QWidget
from spine_items.tool.widgets.tool_spec_optional_widgets import PythonToolSpecOptionalWidget
from spine_items.tool.tool_specifications import PythonTool


class TestPythonToolSpecOptionalWidget(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        """Overridden method. Runs once before all tests in this class."""
        try:
            cls.app = QApplication().processEvents()
        except RuntimeError:
            pass
        logging.basicConfig(
            stream=sys.stderr,
            level=logging.WARNING,
            format="%(asctime)s %(levelname)s: %(message)s",
            datefmt="%Y-%m-%d %H:%M:%S",
        )

    def test_constructor_and_init(self):
        with mock.patch(
            "spine_items.tool.widgets.tool_spec_optional_widgets.PythonToolSpecOptionalWidget._toolbox"
        ) as mock_toolbox:
            mock_toolbox.qsettings.return_value = MockQSettings()
            mock_logger = MagicMock()
            python_tool_spec = PythonTool("a", "python", "", ["fake_main_program.py"], MockQSettings(), mock_logger)
            python_tool_spec.set_execution_settings()  # Sets defaults
            python_tool_spec.execution_settings["executable"] = "fake_python.exe"
            opt_widget = PythonToolSpecOptionalWidget(mock_toolbox)
            opt_widget.init_widget(python_tool_spec)
            self.assertEqual("fake_python.exe", opt_widget.get_executable())
        self.assertIsInstance(opt_widget, PythonToolSpecOptionalWidget)


class MockQSettings:
    def value(self, key, defaultValue=""):
        return defaultValue



######################################################################################################################
# Copyright (C) 2017-2022 Spine project consortium
# This file is part of Spine Items.
# Spine Items is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
# Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
# any later version. This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General
# Public License for more details. You should have received a copy of the GNU Lesser General Public License along with
# this program. If not, see <http://www.gnu.org/licenses/>.
######################################################################################################################

"""
Provides an optional widget for Tool Specification Editor for each Tool Spec type (julia, python, executable, gams).
"""

from PySide6.QtCore import Slot
from PySide6.QtWidgets import QWidget, QApplication
from PySide6.QtGui import Qt, QStandardItemModel, QStandardItem, QIcon
from spine_engine.utils.helpers import resolve_python_interpreter
from spinetoolbox.helpers import file_is_valid, select_python_interpreter
from spinetoolbox.widgets.notification import Notification
from spinetoolbox.kernel_fetcher import KernelFetcher


class OptionalWidget(QWidget):
    def __init__(self, parent):
        """
        Args:
            parent (ToolSpecificationEditorWindow): Tool spec editor window
        """
        super().__init__()
        self._parent = parent

    @property
    def _toolbox(self):
        return self._parent._toolbox

    def init_widget(self, specification):
        raise NotImplementedError

    def add_execution_settings(self):
        raise NotImplementedError


class PythonToolSpecOptionalWidget(OptionalWidget):
    def __init__(self, parent):
        """
        Args:
            parent (ToolSpecificationEditorWindow): Tool spec editor window
        """
        from ..ui.python_kernel_spec_options import Ui_Form  # pylint: disable=import-outside-toplevel

        super().__init__(parent)
        self.ui = Ui_Form()
        self.ui.setupUi(self)
        self.kernel_spec_model = QStandardItemModel(self)
        self.ui.comboBox_kernel_specs.setModel(self.kernel_spec_model)
        self._kernel_spec_editor = None
        self._kernel_spec_model_initialized = False
        self._saved_kernel = None
        self._selected_kernel = None
        self.kernel_fetcher = None
        # Initialize UI elements with defaults
        use_jupyter_console = bool(
            int(self._toolbox.qsettings().value("appSettings/usePythonKernel", defaultValue="0"))
        )
        if use_jupyter_console:
            self.ui.radioButton_jupyter_console.setChecked(True)
            # Get the default kernel spec from Settings->Tools for new Jupyter Console Tool Specs
            self._saved_kernel = self._toolbox.qsettings().value("appSettings/pythonKernel", defaultValue="")
        else:
            self.ui.radioButton_python_console.setChecked(True)
        default_python_path = self._toolbox.qsettings().value("appSettings/pythonPath", defaultValue="")
        self.ui.lineEdit_python_path.setPlaceholderText(resolve_python_interpreter(""))
        self.ui.lineEdit_python_path.setText(default_python_path)
        self.set_ui_for_jupyter_console(use_jupyter_console)
        self.connect_signals()

    def connect_signals(self):
        """Connects signals."""
        self.ui.toolButton_refresh_kernel_specs.clicked.connect(self.start_kernel_fetcher)
        self.ui.comboBox_kernel_specs.activated.connect(self._parent.push_change_kernel_spec_command)
        self.ui.radioButton_jupyter_console.toggled.connect(self._parent.push_set_jupyter_console_mode)
        self.ui.toolButton_browse_python.clicked.connect(self.browse_python_button_clicked)
        self.ui.lineEdit_python_path.editingFinished.connect(self._parent.push_change_executable)
        qApp.aboutToQuit.connect(self.stop_fetching_kernels)  # pylint: disable=undefined-variable

    def init_widget(self, specification):
        """Initializes UI elements based on specification

        Args:
            specification (ToolSpecification): Specification to load
        """
        use_jupyter_console = specification.execution_settings["use_jupyter_console"]
        self.ui.radioButton_jupyter_console.blockSignals(True)
        self.ui.radioButton_python_console.blockSignals(True)
        if use_jupyter_console:
            self.ui.radioButton_jupyter_console.setChecked(True)
        else:
            self.ui.radioButton_python_console.setChecked(True)
        self.ui.radioButton_jupyter_console.blockSignals(False)
        self.ui.radioButton_python_console.blockSignals(False)
        self.set_ui_for_jupyter_console(use_jupyter_console)
        # Must wait until model is built before setting the saved kernel spec as selected
        self._saved_kernel = specification.execution_settings["kernel_spec_name"]
        self.set_executable(specification.execution_settings["executable"])

    def add_execution_settings(self):
        """Collects execution settings based on optional widget state into a dictionary, which is returned."""
        idx = self.ui.comboBox_kernel_specs.currentIndex()
        if idx < 1:
            d = {"kernel_spec_name": "", "env": ""}
        else:
            item = self.ui.comboBox_kernel_specs.model().item(idx)
            k_spec_data = item.data()
            d = k_spec_data
        d["use_jupyter_console"] = self.ui.radioButton_jupyter_console.isChecked()
        self.validate_executable()  # Raises NameError if Python path is not valid
        d["executable"] = self.get_executable()
        return d

    @Slot(bool)
    def browse_python_button_clicked(self, _=False):
        """Calls static method that shows a file browser for selecting a Python interpreter."""
        select_python_interpreter(self, self.ui.lineEdit_python_path)
        self._parent.push_change_executable()

    def set_ui_for_jupyter_console(self, use_jupyter_console):
        """Enables or disables some UI elements in the optional widget according to a checkBox state.

        Args:
            use_jupyter_console (bool): True when Jupyter Console checkBox is checked, false otherwise
        """
        self.ui.lineEdit_python_path.setEnabled(not use_jupyter_console)  # Disable for jupyter console
        self.ui.toolButton_browse_python.setEnabled(not use_jupyter_console)  # Disable for jupyter console
        self.ui.comboBox_kernel_specs.setEnabled(use_jupyter_console)  # Enable for jupyter console
        self.ui.toolButton_refresh_kernel_specs.setEnabled(use_jupyter_console)  # Enable for jupyter console
        if use_jupyter_console and not self._kernel_spec_model_initialized:
            self.start_kernel_fetcher(restore_saved_kernel=True)

    def validate_executable(self):
        """Check that Python path in the line edit is a file it exists and the file name starts with 'python'.

        Raises:
            NameError: If the python path in the line edit is not valid
        """
        p = self.ui.lineEdit_python_path.text().strip()
        if not file_is_valid(self._parent, p, "Invalid Python Interpreter", extra_check="python"):
            raise NameError

    def set_executable(self, p):
        self.ui.lineEdit_python_path.setText(p)

    def get_executable(self):
        return self.ui.lineEdit_python_path.text().strip()

    def find_index_by_data(self, string):
        """Searches the kernel spec model for the first item whose data matches the given string.
        Returns the items row number or -1 if not found."""
        if string == "":
            return 0
        for row in range(1, self.kernel_spec_model.rowCount()):  # Start from index 1
            index = self.kernel_spec_model.index(row, 0)
            item = self.kernel_spec_model.itemFromIndex(index)
            d = item.data()
            if d["kernel_spec_name"] == string:
                return row
        return -1

    @Slot()
    def start_kernel_fetcher(self, restore_saved_kernel=False):
        """Starts KernelFetcher for Python kernels."""
        if self.kernel_fetcher is not None and self.kernel_fetcher.isRunning():
            return
        QApplication.setOverrideCursor(Qt.CursorShape.BusyCursor)
        self.initialize_kernel_spec_model()
        conda_path = self._toolbox.qsettings().value("appSettings/condaPath", defaultValue="")
        self.kernel_fetcher = KernelFetcher(conda_path, fetch_mode=2)
        self.kernel_fetcher.kernel_found.connect(self.add_kernel)
        if restore_saved_kernel:
            self.kernel_fetcher.finished.connect(self._restore_saved_kernel)
        else:
            self.kernel_fetcher.finished.connect(self._restore_selected_kernel)
        self.kernel_fetcher.finished.connect(self._toolbox.restore_override_cursor)
        self.kernel_fetcher.start()

    @Slot()
    def stop_fetching_kernels(self):
        """Terminates the kernel fetcher thread."""
        if self.kernel_fetcher is not None:
            self.kernel_fetcher.stop_fetcher.emit()

    @Slot(str, str, bool, QIcon, dict)
    def add_kernel(self, kernel_name, resource_dir, conda, icon, deats):
        """Adds a kernel entry as an item to kernel spec model."""
        if not conda:
            spec_data = {"kernel_spec_name": kernel_name, "env": ""}
            item = QStandardItem(kernel_name)
        else:
            spec_data = {"kernel_spec_name": kernel_name, "env": "conda"}
            item = QStandardItem(kernel_name + " [Conda]")
        item.setIcon(icon)
        item.setData(spec_data)
        self.kernel_spec_model.appendRow(item)

    def initialize_kernel_spec_model(self):
        """Initializes kernel spec model for fetching kernels. Memorizes currently
        selected kernel in order to select it again after the model has been
        refreshed."""
        item = self.kernel_spec_model.item(self.ui.comboBox_kernel_specs.currentIndex())
        if not item or not item.data():
            self._selected_kernel = None
        else:
            self._selected_kernel = item.data()["kernel_spec_name"]  # Remember the selected kernel spec
        self.kernel_spec_model.clear()
        first_item = QStandardItem("Select kernel spec...")
        self.kernel_spec_model.appendRow(first_item)

    @Slot()
    def _restore_selected_kernel(self):
        """Sets the previously selected kernel spec as the current item after the model has been refreshed."""
        self._kernel_spec_model_initialized = True
        if self._selected_kernel is not None:
            row = self.find_index_by_data(self._selected_kernel)
            if row == -1:
                # The kernel spec may have been removed
                self._parent.push_change_kernel_spec_command(0)
                return
            self.ui.comboBox_kernel_specs.setCurrentIndex(row)

    @Slot()
    def _restore_saved_kernel(self):
        """Sets index of the kernel spec combobox to show the item that was saved with the Tool Specification.
        Make sure this is called after kernel spec model has been populated."""
        self._kernel_spec_model_initialized = True
        if not self._saved_kernel:
            self.ui.comboBox_kernel_specs.setCurrentIndex(0)  # Set 'Select kernel spec...'
        else:
            row = self.find_index_by_data(self._saved_kernel)
            if row == -1:
                notification = Notification(
                    self._parent,
                    f"This Tool spec has kernel spec '{self._saved_kernel}' " f"saved but it could not be found.",
                )
                notification.show()
                row += 1  # Set 'Select kernel spec...'
            self.ui.comboBox_kernel_specs.setCurrentIndex(row)


class ExecutableToolSpecOptionalWidget(OptionalWidget):
    def __init__(self, parent):
        """
        Args:
            parent (QWidget): parent widget
        """
        from ..ui.executable_cmd_exec_options import Ui_Form  # pylint: disable=import-outside-toplevel

        super().__init__(parent)
        self.ui = Ui_Form()
        self.ui.setupUi(self)
        self.shells = ["No shell", "cmd.exe", "powershell.exe", "bash"]
        self.ui.comboBox_shell.addItems(self.shells)
        self.connect_signals()

    def connect_signals(self):
        self.ui.lineEdit_command.editingFinished.connect(self._parent.push_change_executable_command)
        self.ui.comboBox_shell.activated.connect(self._parent.push_change_shell_command)

    def init_widget(self, specification):
        """Initializes UI elements based on specification."""
        self.ui.lineEdit_command.setText(specification.execution_settings["cmd"])
        shell = specification.execution_settings["shell"]
        ind = next(iter(k for k, t in enumerate(self.shells) if t.lower() == shell), 0)
        self.ui.comboBox_shell.setCurrentIndex(ind)

    def add_execution_settings(self):
        """Collects execution settings based on optional widget state into a dictionary, which is returned."""
        return {"cmd": self.ui.lineEdit_command.text(), "shell": self.get_current_shell()}

    def get_current_shell(self):
        """Returns the selected shell in the shell combo box."""
        ind = self.ui.comboBox_shell.currentIndex()
        if ind < 1:
            return ""
        return self.ui.comboBox_shell.currentText()



######################################################################################################################
# Copyright (C) 2017-2022 Spine project consortium
# This file is part of Spine Items.
# Spine Items is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
# Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
# any later version. This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General
# Public License for more details. You should have received a copy of the GNU Lesser General Public License along with
# this program. If not, see <http://www.gnu.org/licenses/>.
######################################################################################################################

"""
Common models.
Contains a generic File list model and an Item for that model.
Used by the Importer and Tool project items but this may be handy for other project items
as well.

"""
from collections import namedtuple
from PySide6.QtCore import QModelIndex, Qt, Signal
from spinetoolbox.mvcmodels.file_list_models import (
    FileListModel,
    CommandLineArgsModel,
    NewCommandLineArgItem,
    CommandLineArgItem,
)
from spine_engine.project_item.project_item_resource import extract_packs


class CheckableFileListModel(FileListModel):
    """A model for checkable files to be shown in a file tree view."""

    FileItem = namedtuple("FileItem", ["resource", "checked"])
    PackItem = namedtuple("PackItem", ["label", "resources", "checked"])

    checked_state_changed = Signal(QModelIndex, bool)
    """Emitted when an item's checked state changes."""

    def data(self, index, role=Qt.ItemDataRole.DisplayRole):
        if role == Qt.ItemDataRole.CheckStateRole:
            if index.internalPointer() is None:
                row = index.row()
                if row < len(self._single_resources):
                    checked = self._single_resources[row].checked
                else:
                    checked = self._pack_resources[row - len(self._single_resources)].checked
                return Qt.CheckState.Checked if checked else Qt.CheckState.Unchecked
            return None
        return super().data(index, role)

    def checked_data(self, index):
        """Returns checked status and label for given top-level index.

        Args:
            index (QModelIndex): top-level index

        Returns:
            tuple: item label and checked flag
        """
        row = index.row()
        if row < len(self._single_resources):
            return self._single_resources[row].resource.label, self._single_resources[row].checked
        pack = self._pack_resources[row - len(self._single_resources)]
        return pack.label, pack.checked

    def is_checked(self, index):
        """Returns True if the item at given index is checked.

        Args:
            index (QModelIndex): index

        Returns:
            bool: True if the item is checked, False otherwise
        """
        pack_label = index.internalPointer()
        if pack_label is None:
            row = index.row()
            if row < len(self._single_resources):
                return self._single_resources[row].checked
            return self._pack_resources[row - len(self._single_resources)].checked
        return self._pack_resources[self._pack_index(pack_label)].checked

    def flags(self, index):
        flags = super().flags(index)
        if index.internalPointer() is None:
            return flags | Qt.ItemIsUserCheckable
        return flags

    def update(self, resources):
        """Updates the model according to given list of resources.

        Args:
            resources (Iterable of ProjectItemResource): resources
        """
        self.beginResetModel()
        unchecked_singles = {item.resource.label for item in self._single_resources if not item.checked}
        unchecked_packs = {item.label for item in self._pack_resources if not item.checked}
        unchecked = unchecked_singles | unchecked_packs
        single_resources, pack_resources = extract_packs(resources)
        new_singles = [self.FileItem(r, r.label not in unchecked) for r in single_resources]
        new_packs = [
            self.PackItem(label, [r for r in r_list if r.hasfilepath], label not in unchecked)
            for label, r_list in pack_resources.items()
        ]
        self._single_resources = new_singles
        self._pack_resources = new_packs
        self.endResetModel()

    def replace(self, old_resource, new_resource):
        """Replaces existing data in the model.

        Note: we don't currently update file pack labels here.

        Args:
            old_resource (ProjectItemResource): resource to replace
            new_resource (ProjectItemResource): new resource
        """
        if old_resource.type_ == "database":
            return
        for row, item in enumerate(self._single_resources):
            if item.resource.provider_name == old_resource.provider_name and item.resource.label == old_resource.label:
                self._single_resources[row] = item._replace(resource=new_resource)
                index = self.index(row, 0)
                self.dataChanged.emit(index, index, [Qt.ItemDataRole.DisplayRole, Qt.ItemDataRole.ToolTipRole])
                return
        for pack_row, pack in enumerate(self._pack_resources):
            for row, resource in enumerate(pack.resources):
                if resource.provider_name == old_resource.provider_name and resource.label == old_resource.label:
                    if new_resource.label == resource.label:
                        pack.resources[row] = new_resource
                        pack_index = self.index(pack_row, 0)
                        index = self.index(row, 0, pack_index)
                        self.dataChanged.emit(index, index, [Qt.ItemDataRole.DisplayRole, Qt.ItemDataRole.ToolTipRole])
                        return
                    else:
                        single_resources = [item.resource for item in self._single_resources]
                        new_pack_resources = [
                            r
                            for existing_pack in self._pack_resources
                            for r in existing_pack.resources
                            if r is not resource
                        ]
                        new_pack_resources.append(new_resource)
                        self.update(single_resources + new_pack_resources)
                        return

    def setData(self, index, value, role=Qt.ItemDataRole.EditRole):
        """Sets data in the model."""
        if role != Qt.ItemDataRole.CheckStateRole or not index.isValid():
            return False
        checked = value == Qt.CheckState.Checked.value
        self.checked_state_changed.emit(index, checked)
        return True

    def set_initial_state(self, selected_items):
        """Fills model with incomplete data; needs a call to :func:`update` to make the model usable.

        Args:
            selected_items (dict): mapping from item label to checked flag
        """
        for label, selected in selected_items.items():
            self._pack_resources.append(self.PackItem(label, [], selected))

    def set_checked(self, index, checked):
        """Checks or unchecks given item.

        Args:
            index (QModelIndex): resource label
            checked (bool): checked flag
        """
        row = index.row()
        if row < len(self._single_resources):
            self._single_resources[row] = self._single_resources[row]._replace(checked=checked)
        else:
            row -= len(self._single_resources)
            self._pack_resources[row] = self._pack_resources[row]._replace(checked=checked)
        self.dataChanged.emit(index, index, [Qt.ItemDataRole.CheckStateRole])

    def index_with_file_path(self):
        """Tries to find an item that has a valid file path or URL.

        Returns:
            QModelIndex: index to an item with usable file path/URL, or an invalid index if none found
        """
        for row, item in enumerate(self._single_resources):
            if item.checked and item.resource.url:
                return self.index(row, 0)
        for pack_row, item in enumerate(self._pack_resources):
            if item.checked:
                for row, resource in enumerate(item.resources):
                    if resource.hasfilepath:
                        return self.index(row, 0, self.index(pack_row, 0))
        return QModelIndex()


class ToolCommandLineArgsModel(CommandLineArgsModel):
    def __init__(self, parent=None):
        super().__init__(parent)
        self._spec_args_root = CommandLineArgItem("Specification arguments")
        self._tool_args_root = CommandLineArgItem("Tool arguments", drop_enabled=True)
        self.appendRow(self._spec_args_root)
        self.appendRow(self._tool_args_root)
        self._tool_args_root.appendRow(NewCommandLineArgItem())

    def reset_model(self, spec_args, tool_args):
        self._args = tool_args
        self._reset_root(self._spec_args_root, spec_args, dict(), has_empty_row=False)
        self._reset_root(
            self._tool_args_root, tool_args, dict(editable=True, selectable=True, drag_enabled=True), has_empty_row=True
        )

    def canDropMimeData(self, data, drop_action, row, column, parent):
        return parent.data() is not None and row >= 0



######################################################################################################################
# Copyright (C) 2017-2022 Spine project consortium
# This file is part of Spine Items.
# Spine Items is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
# Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
# any later version. This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General
# Public License for more details. You should have received a copy of the GNU Lesser General Public License along with
# this program. If not, see <http://www.gnu.org/licenses/>.
######################################################################################################################

"""
Unit tests for :literal:`utils` module.

"""
from hashlib import sha1
from pathlib import Path
from tempfile import TemporaryDirectory
import unittest
from spine_items.tool.utils import find_last_output_files


class TestFindLastOutputFiles(unittest.TestCase):
    def setUp(self):
        self._temp_dir = TemporaryDirectory()

    def tearDown(self):
        self._temp_dir.cleanup()

    def test_returns_empty_results_with_no_output_files_and_empty_output_dir(self):
        files = find_last_output_files([], self._temp_dir.name)
        self.assertEqual(files, {})

    def test_finds_output_file(self):
        stamped_dir = Path(self._temp_dir.name) / "2021-11-17T11.00.00"
        stamped_dir.mkdir()
        data_file = stamped_dir / "data.dat"
        data_file.touch()
        files = find_last_output_files(["data.dat"], self._temp_dir.name)
        self.assertEqual(files, {"data.dat": [str(data_file)]})

    def test_finds_output_file_in_subdirectory(self):
        stamped_dir = Path(self._temp_dir.name) / "2021-11-17T11.00.00"
        stamped_dir.mkdir()
        out_dir = stamped_dir / "out"
        out_dir.mkdir()
        data_file = out_dir / "data.dat"
        data_file.touch()
        files = find_last_output_files(["out/data.dat"], self._temp_dir.name)
        self.assertEqual(files, {"out/data.dat": [str(data_file)]})

    def test_finds_file_pattern(self):
        stamped_dir = Path(self._temp_dir.name) / "2021-11-17T11.00.00"
        stamped_dir.mkdir()
        data_file = stamped_dir / "data.dat"
        data_file.touch()
        files = find_last_output_files(["*.dat"], self._temp_dir.name)
        self.assertEqual(files, {"*.dat": [str(data_file)]})

    def test_finds_file_pattern_in_subdirectory(self):
        stamped_dir = Path(self._temp_dir.name) / "2021-11-17T11.00.00"
        stamped_dir.mkdir()
        out_dir = stamped_dir / "out"
        out_dir.mkdir()
        data_file = out_dir / "data.dat"
        data_file.touch()
        files = find_last_output_files(["out/*.dat"], self._temp_dir.name)
        self.assertEqual(files, {"out/*.dat": [str(data_file)]})

    def test_ignore_failed_directory(self):
        failed_dir = Path(self._temp_dir.name) / "failed"
        failed_dir.mkdir()
        data_file = failed_dir / "data.dat"
        data_file.touch()
        files = find_last_output_files(["data.dat"], self._temp_dir.name)
        self.assertEqual(files, {})

    def test_finds_latest_output_directory(self):
        stamped_dir_old = Path(self._temp_dir.name) / "2021-11-17T11.00.00"
        stamped_dir_old.mkdir()
        data_file_old = stamped_dir_old / "data.dat"
        data_file_old.touch()
        stamped_dir_new = Path(self._temp_dir.name) / "2021-11-17T11.30.00"
        stamped_dir_new.mkdir()
        data_file_new = stamped_dir_new / "data.dat"
        data_file_new.touch()
        files = find_last_output_files(["data.dat"], self._temp_dir.name)
        self.assertEqual(files, {"data.dat": [str(data_file_new)]})

    def test_finds_latest_output_directory_within_filter_id_directories(self):
        filter_id_dir = Path(self._temp_dir.name) / sha1(b"filter_id").hexdigest()
        filter_id_dir.mkdir()
        stamped_dir_old = filter_id_dir / "2021-11-17T11.00.00"
        stamped_dir_old.mkdir()
        data_file_old = stamped_dir_old / "data.dat"
        data_file_old.touch()
        stamped_dir_new = filter_id_dir / "2021-11-17T11.30.00"
        stamped_dir_new.mkdir()
        data_file_new = stamped_dir_new / "data.dat"
        data_file_new.touch()
        files = find_last_output_files(["data.dat"], self._temp_dir.name)
        self.assertEqual(files, {"data.dat": [str(data_file_new)]})

    def test_finds_latest_output_directory_within_filter_id_directories_with_mixed_directories(self):
        stamped_dir_old = Path(self._temp_dir.name) / "2021-11-17T11.00.00"
        stamped_dir_old.mkdir()
        data_file_old = stamped_dir_old / "data.dat"
        data_file_old.touch()
        filter_id_dir = Path(self._temp_dir.name) / sha1(b"filter_id").hexdigest()
        filter_id_dir.mkdir()
        stamped_dir_new = filter_id_dir / "2021-11-17T11.30.00"
        stamped_dir_new.mkdir()
        data_file_new = stamped_dir_new / "data.dat"
        data_file_new.touch()
        files = find_last_output_files(["data.dat"], self._temp_dir.name)
        self.assertEqual(files, {"data.dat": [str(data_file_new)]})

    def test_finds_latest_output_directory_within_non_filter_id_directories_with_mixed_directories(self):
        filter_id_dir = Path(self._temp_dir.name) / sha1(b"filter_id").hexdigest()
        filter_id_dir.mkdir()
        stamped_dir_old = filter_id_dir / "2021-11-17T11.00.00"
        stamped_dir_old.mkdir()
        data_file_old = stamped_dir_old / "data.dat"
        data_file_old.touch()
        stamped_dir_new = Path(self._temp_dir.name) / "2021-11-17T11.30.00"
        stamped_dir_new.mkdir()
        data_file_new = stamped_dir_new / "data.dat"
        data_file_new.touch()
        files = find_last_output_files(["data.dat"], self._temp_dir.name)
        self.assertEqual(files, {"data.dat": [str(data_file_new)]})


if __name__ == '__main__':
    unittest.main()



######################################################################################################################
# Copyright (C) 2017-2022 Spine project consortium
# This file is part of Spine Items.
# Spine Items is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
# Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
# any later version. This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General
# Public License for more details. You should have received a copy of the GNU Lesser General Public License along with
# this program. If not, see <http://www.gnu.org/licenses/>.
######################################################################################################################

"""
Data store properties widget.

"""

from spinetoolbox.widgets.properties_widget import PropertiesWidgetBase
from spinedb_api import SUPPORTED_DIALECTS
from ...widgets import UrlSelectorMixin


class DataStorePropertiesWidget(UrlSelectorMixin, PropertiesWidgetBase):
    """Widget for the Data Store Item Properties."""

    def __init__(self, toolbox):
        """
        Args:
            toolbox (ToolboxUI): The toolbox instance where this widget should be embedded
        """
        from ..ui.data_store_properties import Ui_Form  # pylint: disable=import-outside-toplevel

        super().__init__(toolbox)
        self._active_item = None
        self.ui = Ui_Form()
        self.ui.setupUi(self)
        self.ui.url_selector_widget.setup(
            list(SUPPORTED_DIALECTS.keys()), self._select_sqlite_file, True, self._toolbox
        )

    def set_item(self, data_store):
        """Sets the active project item for the properties widget.

        Args:
            data_store (DataStore): data store
        """
        self._active_item = data_store

    def unset_item(self):
        """Unsets the active item."""
        self._active_item = None

    def _select_sqlite_file(self):
        """Lets active item select an SQLite file.

        Returns:
            str: file path or None if operation was cancelled
        """
        if self._active_item is None:
            return None
        return self._active_item.select_sqlite_file()



######################################################################################################################
# Copyright (C) 2017-2022 Spine project consortium
# This file is part of Spine Toolbox.
# Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
# Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
# any later version. This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General
# Public License for more details. You should have received a copy of the GNU Lesser General Public License along with
# this program. If not, see <http://www.gnu.org/licenses/>.
######################################################################################################################

"""
Contains unit tests for the tool_instance module.

"""
import sys
import unittest
from unittest import mock

from spine_engine.execution_managers.persistent_execution_manager import kill_persistent_processes
from spine_items.tool.tool_specifications import PythonTool


class TestPythonToolInstance(unittest.TestCase):
    @classmethod
    def tearDownClass(cls):
        kill_persistent_processes()

    def test_prepare_with_cmd_line_arguments_in_jupyter_kernel(self):
        instance = self._make_tool_instance(True)
        with mock.patch("spine_items.tool.tool_instance.KernelExecutionManager"):
            instance.prepare(["arg1", "arg2"])
        self.assertEqual(instance.args, ['%cd -q path/', '%run "main.py" "arg1" "arg2"'])

    def test_prepare_with_empty_cmd_line_arguments_in_jupyter_kernel(self):
        instance = self._make_tool_instance(True)
        with mock.patch("spine_items.tool.tool_instance.KernelExecutionManager"):
            instance.prepare([])
        self.assertEqual(instance.args, ['%cd -q path/', '%run "main.py"'])

    def test_prepare_with_cmd_line_arguments_in_persistent_process(self):
        instance = self._make_tool_instance(False)
        with mock.patch("spine_engine.execution_managers.persistent_execution_manager.PersistentManagerBase"):
            instance.prepare(["arg1", "arg2"])
        self.assertEqual(instance.program, [sys.executable])
        self.assertEqual(instance.exec_mngr.alias, "python main.py arg1 arg2")
        instance.terminate_instance()

    def test_prepare_without_cmd_line_arguments_in_persistent_process(self):
        instance = self._make_tool_instance(False)
        with mock.patch("spine_engine.execution_managers.persistent_execution_manager.PersistentManagerBase"):
            instance.prepare([])
        self.assertEqual(instance.program, [sys.executable])
        self.assertEqual(instance.exec_mngr.alias, "python main.py")
        instance.terminate_instance()

    @staticmethod
    def _make_tool_instance(execute_in_embedded_console):
        settings = mock.NonCallableMagicMock()
        if execute_in_embedded_console:
            settings.value = mock.MagicMock(return_value="2")
        else:

            def get_setting(name, defaultValue):
                return {"appSettings/pythonPath": sys.executable, "appSettings/useEmbeddedPython": "0"}.get(
                    name, defaultValue
                )

            settings.value = mock.MagicMock(side_effect=get_setting)
        logger = mock.Mock()
        path = ""
        source_files = ["main.py"]
        specification = PythonTool("specification name", "python", path, source_files, settings, logger)
        base_directory = "path/"
        return specification.create_tool_instance(base_directory, False, logger, mock.Mock())


if __name__ == '__main__':
    unittest.main()



######################################################################################################################
# Copyright (C) 2017-2022 Spine project consortium
# This file is part of Spine Items.
# Spine Items is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
# Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
# any later version. This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General
# Public License for more details. You should have received a copy of the GNU Lesser General Public License along with
# this program. If not, see <http://www.gnu.org/licenses/>.
######################################################################################################################

"""
Unit tests for Tool project item.

"""

from tempfile import TemporaryDirectory
import unittest
from unittest import mock
import os
from PySide6.QtGui import QStandardItem, QStandardItemModel
from PySide6.QtWidgets import QApplication, QMenu
from spine_items.tool.item_info import ItemInfo
from spine_items.tool.tool_specifications import ExecutableTool
from spine_items.tool.tool import Tool
from spine_items.tool.tool_factory import ToolFactory
from spine_engine.project_item.project_item_resource import ProjectItemResource
from spine_engine.config import TOOL_OUTPUT_DIR
from tests.mock_helpers import mock_finish_project_item_construction, create_mock_project, create_mock_toolbox


class TestTool(unittest.TestCase):
    def setUp(self):
        """Set up."""
        self._temp_dir = TemporaryDirectory()
        self.toolbox = create_mock_toolbox()
        self.project = create_mock_project(self._temp_dir.name)
        specifications = [
            ExecutableTool(
                name="simple_exec",
                tooltype="executable",
                path=self._temp_dir.name,
                includes=["main.sh"],
                settings=self.toolbox.qsettings(),
                logger=self.toolbox,
                description="A simple executable tool.",
                inputfiles=["input1.csv", "input2.csv"],
                inputfiles_opt=["opt_input.csv"],
                outputfiles=["output1.csv", "output2.csv"],
                cmdline_args="<args>",
            ),
            ExecutableTool(
                name="complex_exec",
                tooltype="executable",
                path=self._temp_dir.name,
                includes=["MakeFile", "src/a.c", "src/a.h", "src/subunit/x.c", "src/subunit/x.h"],
                settings=self.toolbox.qsettings(),
                logger=self.toolbox,
                description="A more complex executable tool.",
                inputfiles=["input1.csv", "input/input2.csv"],
                inputfiles_opt=["opt/*.ini", "?abc.txt"],
                outputfiles=["output1.csv", "output/output2.csv"],
                cmdline_args="subunit",
            ),
        ]
        self.specification_dict = {x.name: x for x in specifications}
        self.project.get_specification = self.specification_dict.get
        self.toolbox.project.return_value = self.project
        self.model = _MockToolSpecModel(self.specification_dict)

    def tearDown(self):
        self._temp_dir.cleanup()

    @classmethod
    def setUpClass(cls):
        if not QApplication.instance():
            QApplication()

    def test_item_type(self):
        self.assertEqual(Tool.item_type(), ItemInfo.item_type())

    def test_item_category(self):
        self.assertEqual(Tool.item_category(), ItemInfo.item_category())

    def test_item_dict(self):
        """Tests Item dictionary creation."""
        tool = self._add_tool()
        d = tool.item_dict()
        a = ["type", "description", "x", "y", "specification", "execute_in_work", "cmd_line_args"]
        for k in a:
            self.assertTrue(k in d, f"Key '{k}' not in dict {d}")

    def test_notify_destination(self):
        source_item = mock.NonCallableMagicMock()
        source_item.name = "source name"
        source_item.item_type = mock.MagicMock(return_value="Data Connection")
        tool = self._add_tool()
        tool.logger.msg = mock.MagicMock()
        tool.logger.msg_warning = mock.MagicMock()
        tool.notify_destination(source_item)
        tool.logger.msg.emit.assert_called_with(
            "Link established. Tool <b>T</b> will look for input "
            "files from <b>source name</b>'s references and data directory."
        )
        source_item.item_type = mock.MagicMock(return_value="Importer")
        tool.notify_destination(source_item)
        tool.logger.msg_warning.emit.assert_called_with(
            "Link established. Interaction between a " "<b>Importer</b> and a <b>Tool</b> has not been implemented yet."
        )
        source_item.item_type = mock.MagicMock(return_value="Data Store")
        tool.notify_destination(source_item)
        tool.logger.msg.emit.assert_called_with(
            "Link established. Data Store <b>source name</b> url will " "be passed to Tool <b>T</b> when executing."
        )
        source_item.item_type = mock.MagicMock(return_value="Exporter")
        tool.notify_destination(source_item)
        tool.logger.msg.emit.assert_called_with(
            "Link established. The file exported by <b>source name</b> are now available in <b>T</b>."
        )
        source_item.item_type = mock.MagicMock(return_value="Tool")
        tool.notify_destination(source_item)
        tool.logger.msg.emit.assert_called_with("Link established")
        source_item.item_type = mock.MagicMock(return_value="View")
        tool.notify_destination(source_item)
        tool.logger.msg_warning.emit.assert_called_with(
            "Link established. Interaction between a " "<b>View</b> and a <b>Tool</b> has not been implemented yet."
        )

    def test_rename(self):
        """Tests renaming a self.tool."""
        tool = self._add_tool()
        tool.activate()
        expected_name = "ABC"
        expected_short_name = "abc"
        tool.rename(expected_name, "")
        # Check name
        self.assertEqual(expected_name, tool.name)  # item name
        self.assertEqual(expected_name, tool.get_icon().name_item.text())  # name item on Design View
        # Check data_dir
        expected_data_dir = os.path.join(self.project.items_dir, expected_short_name)
        self.assertEqual(expected_data_dir, tool.data_dir)  # Check data dir
        # Check that output_dir has been updated
        expected_output_dir = os.path.join(tool.data_dir, TOOL_OUTPUT_DIR)
        self.assertEqual(expected_output_dir, tool.output_dir)

    def test_load_tool_specification(self):
        """Test that specification is loaded into selections on Tool creation,
        and then shown in the ui when Tool is activated.
        """
        item_dict = {
            "type": "Tool",
            "description": "",
            "x": 0,
            "y": 0,
            "specification": "simple_exec",
            "execute_in_work": False,
        }
        tool = self._add_tool(item_dict)
        with mock.patch("spine_items.tool.tool.ToolSpecificationMenu") as mock_tool_spec_menu:
            mock_tool_spec_menu.side_effect = lambda *args: QMenu()
            tool.activate()
            self._assert_is_simple_exec_tool(tool)

    def test_save_and_restore_selections(self):
        """Test that selections are saved and restored when deactivating a Tool and activating it again."""
        item_dict = {"type": "Tool", "description": "", "x": 0, "y": 0, "specification": ""}
        tool = self._add_tool(item_dict)
        with mock.patch("spine_items.tool.tool.ToolSpecificationMenu") as mock_tool_spec_menu:
            mock_tool_spec_menu.side_effect = lambda *args: QMenu()
            tool.activate()
            self._assert_is_no_tool(tool)
            # Set the simple_exec tool specification manually
            tool._properties_ui.comboBox_tool.textActivated.emit("simple_exec")
            tool._properties_ui.radioButton_execute_in_source.setChecked(True)
            self._assert_is_simple_exec_tool(tool)
            tool.deactivate()
            tool.activate()
            self._assert_is_simple_exec_tool(tool)

    def test_find_input_files(self):
        """Test that input files are correctly found in resources
        when required input files and available resources are updated"""
        separator = os.sep
        item_dict = {"type": "Tool", "description": "", "x": 0, "y": 0, "specification": "simple_exec"}
        tool = self._add_tool(item_dict)
        url1 = os.path.join(self._temp_dir.name, "more_files", "input1.csv")
        url2 = os.path.join(self._temp_dir.name, "more_files", "data.csv")
        url3 = os.path.join(self._temp_dir.name, "more filess", "input1.csv")
        url4 = os.path.join(self._temp_dir.name, "other.csv")
        url5 = os.path.join(self._temp_dir.name, "other", "input2.csv")
        url6 = os.path.join(self._temp_dir.name, "input3.csv")
        expected_urls = {"url1": url1, "url2": url2, "url3": url3, "url4": url4, "url5": url5, "url6": url6}
        for key, url in expected_urls.items():
            i = url.find(separator)
            expected_urls[key] = url[:i] + separator + url[i:]
        resources = [
            ProjectItemResource("Exporter", "file", "first", url="file:///" + url1, metadata={}, filterable=False),
            ProjectItemResource("Exporter", "url", "second", url="file:///" + url2, metadata={}, filterable=False),
            ProjectItemResource("Exporter", "file", "third", url="file:///" + url3, metadata={}, filterable=False),
            ProjectItemResource("Exporter", "url", "fourth", url="file:///" + url4, metadata={}, filterable=False),
        ]
        result = tool._find_input_files(resources)
        expected = {"input1.csv": [expected_urls["url1"], expected_urls["url3"]], "input2.csv": None}
        self.assertEqual(2, len(result))
        self.assertEqual(expected["input2.csv"], result["input2.csv"])
        self.assertTrue(expected_urls["url3"] in result["input1.csv"] or expected_urls["url1"] in result["input1.csv"])
        resources.pop(0)
        resources.append(
            ProjectItemResource("Exporter", "file", "fifth", url="file:///" + url5, metadata={}, filterable=False)
        )
        result = tool._find_input_files(resources)
        expected = {'input2.csv': [expected_urls["url5"]], 'input1.csv': [expected_urls["url3"]]}
        self.assertEqual(expected, result)
        resources.append(
            ProjectItemResource("Exporter", "file", "sixth", url="file:///" + url6, metadata={}, filterable=False)
        )
        tool.specification().inputfiles = set(["input2.csv", os.path.join(self._temp_dir.name, "input3.csv")])
        result = tool._find_input_files(resources)
        expected = {
            os.path.join(self._temp_dir.name, "input3.csv"): [expected_urls["url6"]],
            'input2.csv': [expected_urls["url5"]],
        }
        self.assertEqual(expected, result)

    def _add_tool(self, item_dict=None):
        if item_dict is None:
            item_dict = {"type": "Tool", "description": "", "x": 0, "y": 0}
        factory = ToolFactory()
        tool = factory.make_item("T", item_dict, self.toolbox, self.project)
        self._properties_widget = mock_finish_project_item_construction(factory, tool, self.toolbox)
        # Set model for tool combo box
        tool._properties_ui.comboBox_tool.setModel(self.model)
        return tool

    def _assert_is_simple_exec_tool(self, tool):
        """Assert that the given tool has the simple_exec specification."""
        # Check tool spec
        source_files = tool.specification().includes
        input_files = tool.specification().inputfiles
        opt_input_files = tool.specification().inputfiles_opt
        output_files = tool.specification().outputfiles
        self.assertEqual(source_files, ["main.sh"])
        self.assertIn("input1.csv", input_files)
        self.assertIn("input2.csv", input_files)
        self.assertEqual(opt_input_files, {"opt_input.csv"})
        self.assertIn("output1.csv", output_files)
        self.assertIn("output2.csv", output_files)
        # Check cmdline model
        spec_args_root = tool._cmdline_args_model.invisibleRootItem().child(0)
        self.assertEqual(spec_args_root.rowCount(), 1)
        cmdline_args = spec_args_root.child(0).text()
        self.assertEqual(cmdline_args, "<args>")
        # Check ui
        combox_text = tool._properties_ui.comboBox_tool.currentText()
        in_work = tool._properties_ui.radioButton_execute_in_work.isChecked()
        in_source = tool._properties_ui.radioButton_execute_in_source.isChecked()
        self.assertEqual(combox_text, "simple_exec")
        self.assertFalse(in_work)
        self.assertTrue(in_source)

    def _assert_is_no_tool(self, tool):
        """Assert that the given tool has no tool specification."""
        # Check cmdline model
        tool_args_root = tool._cmdline_args_model.invisibleRootItem().child(1)
        self.assertEqual(tool_args_root.rowCount(), 1)  # The "Type new arg here..." item
        # Check ui
        combox_text = tool._properties_ui.comboBox_tool.currentText()
        in_work = tool._properties_ui.radioButton_execute_in_work.isChecked()
        in_source = tool._properties_ui.radioButton_execute_in_source.isChecked()
        self.assertEqual(combox_text, "")
        self.assertTrue(in_work)
        self.assertFalse(in_source)


class _MockToolSpecModel(QStandardItemModel):
    # Create a dictionary of tool specifications to 'populate' the mock model
    def __init__(self, specifications):
        super().__init__()
        self._specifications = specifications
        self.invisibleRootItem().appendRows([QStandardItem(x) for x in self._specifications])


if __name__ == "__main__":
    unittest.main()



######################################################################################################################
# Copyright (C) 2017-2022 Spine project consortium
# This file is part of Spine Items.
# Spine Items is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
# Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
# any later version. This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General
# Public License for more details. You should have received a copy of the GNU Lesser General Public License along with
# this program. If not, see <http://www.gnu.org/licenses/>.
######################################################################################################################

"""
Unit tests for ToolSpecificationEditorWindow class.

"""

import unittest
import logging
import sys
from unittest import mock
from tempfile import NamedTemporaryFile
from pathlib import Path
from PySide6.QtWidgets import QApplication
from spine_items.tool.widgets.tool_specification_editor_window import ToolSpecificationEditorWindow
from tests.mock_helpers import create_mock_toolbox


class TestToolSpecificationEditorWindow(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        """Overridden method. Runs once before all tests in this class."""
        try:
            cls.app = QApplication().processEvents()
        except RuntimeError:
            pass
        logging.basicConfig(
            stream=sys.stderr,
            level=logging.DEBUG,
            format="%(asctime)s %(levelname)s: %(message)s",
            datefmt="%Y-%m-%d %H:%M:%S",
        )

    def setUp(self):
        """Overridden method. Runs before each test."""
        self.toolbox = create_mock_toolbox()
        with mock.patch("spinetoolbox.project_item.specification_editor_window.restore_ui"):
            self.tool_specification_widget = ToolSpecificationEditorWindow(self.toolbox)

    def tearDown(self):
        """Overridden method. Runs after each test.
        Use this to free resources after a test if needed.
        """
        self.tool_specification_widget.deleteLater()
        self.tool_specification_widget = None

    def test_create_minimal_julia_tool_specification(self):
        self.tool_specification_widget._ui.comboBox_tooltype.setCurrentIndex(0)  # 0: Julia
        self.tool_specification_widget._spec_toolbar._line_edit_name.setText("test_tool")
        with NamedTemporaryFile(mode="r") as temp_file:
            self.tool_specification_widget._set_main_program_file(str(Path(temp_file.name)))
            self.tool_specification_widget._save()

    def test_create_minimal_gams_tool_specification(self):
        self.tool_specification_widget._ui.comboBox_tooltype.setCurrentIndex(2)  # 2: gams
        self.tool_specification_widget._spec_toolbar._line_edit_name.setText("test_tool")
        with NamedTemporaryFile(mode="r") as temp_file:
            self.tool_specification_widget._set_main_program_file(str(Path(temp_file.name)))
            self.tool_specification_widget._save()

    def test_create_minimal_executable_tool_specification(self):
        self.tool_specification_widget._ui.comboBox_tooltype.setCurrentIndex(3)  # 3: executable
        self.tool_specification_widget._spec_toolbar._line_edit_name.setText("test_tool")
        with NamedTemporaryFile(mode="r") as temp_file:
            self.tool_specification_widget._set_main_program_file(str(Path(temp_file.name)))
            self.tool_specification_widget._save()

    # @unittest.skip("Obsolete")
    # def test_add_cmdline_tag_url_inputs(self):
    #     self._test_add_cmdline_tag_on_empty_args_field("@@url_inputs@@")
    #
    # @unittest.skip("Obsolete")
    # def test_add_cmdline_tag_url_inputs_middle_of_other_tags(self):
    #     self._test_add_cmdline_tag_middle_of_other_tags("@@url_inputs@@")
    #
    # @unittest.skip("Obsolete")
    # def test_add_cmdline_tag_url_inputs_no_space_before_regular_arg(self):
    #     self._test_add_cmdline_tag_adds_no_space_before_regular_arg("@@url_inputs@@")
    #
    # @unittest.skip("Obsolete")
    # def test_add_cmdline_tag_url_outputs(self):
    #     self._test_add_cmdline_tag_on_empty_args_field("@@url_outputs@@")
    #
    # @unittest.skip("Obsolete")
    # def test_add_cmdline_tag_url_outputs_middle_of_other_tags(self):
    #     self._test_add_cmdline_tag_middle_of_other_tags("@@url_outputs@@")
    #
    # @unittest.skip("Obsolete")
    # def test_add_cmdline_tag_url_outputs_no_space_before_regular_arg(self):
    #     self._test_add_cmdline_tag_adds_no_space_before_regular_arg("@@url_outputs@@")
    #
    # @unittest.skip("Obsolete")
    # def test_add_cmdline_tag_data_store_url(self):
    #     self._test_add_cmdline_tag_on_empty_args_field("@@url:<data-store-name>@@")
    #     selection = self.tool_specification_widget.ui.lineEdit_args.selectedText()
    #     self.assertEqual(selection, "<data-store-name>")
    #
    # @unittest.skip("Obsolete")
    # def test_add_cmdline_tag_data_store_url_middle_of_other_tags(self):
    #     self._test_add_cmdline_tag_middle_of_other_tags("@@url:<data-store-name>@@")
    #     selection = self.tool_specification_widget.ui.lineEdit_args.selectedText()
    #     self.assertEqual(selection, "<data-store-name>")
    #
    # @unittest.skip("Obsolete")
    # def test_add_cmdline_tag_data_store_url_no_space_before_regular_arg(self):
    #     self._test_add_cmdline_tag_adds_no_space_before_regular_arg("@@url:<data-store-name>@@")
    #     selection = self.tool_specification_widget.ui.lineEdit_args.selectedText()
    #     self.assertEqual(selection, "<data-store-name>")
    #
    # @unittest.skip("Obsolete")
    # def test_add_cmdline_tag_optional_inputs(self):
    #     self._test_add_cmdline_tag_on_empty_args_field("@@optional_inputs@@")
    #
    # @unittest.skip("Obsolete")
    # def test_add_cmdline_tag_optional_inputs_middle_of_other_tags(self):
    #     self._test_add_cmdline_tag_middle_of_other_tags("@@optional_inputs@@")
    #
    # @unittest.skip("Obsolete")
    # def test_add_cmdline_tag_optional_inputs_no_space_before_regular_arg(self):
    #     self._test_add_cmdline_tag_adds_no_space_before_regular_arg("@@optional_inputs@@")
    #
    # def _find_action(self, action_text, actions):
    #     found_action = None
    #     for action in actions:
    #         if action.text() == action_text:
    #             found_action = action
    #             break
    #     self.assertIsNotNone(found_action)
    #     return found_action
    #
    # def _test_add_cmdline_tag_on_empty_args_field(self, tag):
    #     menu = self.tool_specification_widget.ui.toolButton_add_cmdline_tag.menu()
    #     url_inputs_action = self._find_action(tag, menu.actions())
    #     url_inputs_action.trigger()
    #     args = self.tool_specification_widget.ui.lineEdit_args.text()
    #     expected = tag + " "
    #     self.assertEqual(args, expected)
    #     if not self.tool_specification_widget.ui.lineEdit_args.hasSelectedText():
    #         cursor_position = self.tool_specification_widget.ui.lineEdit_args.cursorPosition()
    #         self.assertEqual(cursor_position, len(expected))
    #
    # def _test_add_cmdline_tag_middle_of_other_tags(self, tag):
    #     self.tool_specification_widget.ui.lineEdit_args.setText("@@optional_inputs@@@@url_outputs@@")
    #     self.tool_specification_widget.ui.lineEdit_args.setCursorPosition(len("@@optional_inputs@@"))
    #     menu = self.tool_specification_widget.ui.toolButton_add_cmdline_tag.menu()
    #     url_inputs_action = self._find_action(tag, menu.actions())
    #     url_inputs_action.trigger()
    #     args = self.tool_specification_widget.ui.lineEdit_args.text()
    #     self.assertEqual(args, f"@@optional_inputs@@ {tag} @@url_outputs@@")
    #     if not self.tool_specification_widget.ui.lineEdit_args.hasSelectedText():
    #         cursor_position = self.tool_specification_widget.ui.lineEdit_args.cursorPosition()
    #         self.assertEqual(cursor_position, len(f"@@optional_inputs@@ {tag} "))
    #
    # def _test_add_cmdline_tag_adds_no_space_before_regular_arg(self, tag):
    #     self.tool_specification_widget.ui.lineEdit_args.setText("--tag=")
    #     self.tool_specification_widget.ui.lineEdit_args.setCursorPosition(len("--tag="))
    #     menu = self.tool_specification_widget.ui.toolButton_add_cmdline_tag.menu()
    #     url_inputs_action = self._find_action(tag, menu.actions())
    #     url_inputs_action.trigger()
    #     args = self.tool_specification_widget.ui.lineEdit_args.text()
    #     self.assertEqual(args, f"--tag={tag} ")
    #     if not self.tool_specification_widget.ui.lineEdit_args.hasSelectedText():
    #         cursor_position = self.tool_specification_widget.ui.lineEdit_args.cursorPosition()
    #         self.assertEqual(cursor_position, len(f"--tag={tag} "))


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
######################################################################################################################
# Copyright (C) 2017-2022 Spine project consortium
# This file is part of Spine Items.
# Spine Items is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
# Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
# any later version. This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General
# Public License for more details. You should have received a copy of the GNU Lesser General Public License along with
# this program. If not, see <http://www.gnu.org/licenses/>.
######################################################################################################################

################################################################################
## Form generated from reading UI file 'import_editor_window.ui'
##
## Created by: Qt User Interface Compiler version 6.5.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QAction, QBrush, QColor, QConicalGradient,
    QCursor, QFont, QFontDatabase, QGradient,
    QIcon, QImage, QKeySequence, QLinearGradient,
    QPainter, QPalette, QPixmap, QRadialGradient,
    QTransform)
from PySide6.QtWidgets import (QAbstractItemView, QApplication, QCheckBox, QComboBox,
    QDockWidget, QFormLayout, QHBoxLayout, QHeaderView,
    QLabel, QListView, QMainWindow, QPushButton,
    QSizePolicy, QSpacerItem, QSpinBox, QTableView,
    QToolButton, QVBoxLayout, QWidget)

from spine_items.importer.widgets.multi_checkable_list_view import MultiCheckableListView
from spine_items.importer.widgets.table_view_with_button_header import TableViewWithButtonHeader
from spinetoolbox.widgets.custom_combobox import ElidedCombobox
from spine_items import resources_icons_rc

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        if not MainWindow.objectName():
            MainWindow.setObjectName(u"MainWindow")
        MainWindow.resize(964, 665)
        MainWindow.setDockNestingEnabled(True)
        self.export_mappings_action = QAction(MainWindow)
        self.export_mappings_action.setObjectName(u"export_mappings_action")
        self.export_mappings_action.setEnabled(False)
        self.import_mappings_action = QAction(MainWindow)
        self.import_mappings_action.setObjectName(u"import_mappings_action")
        self.import_mappings_action.setEnabled(False)
        self.actionSwitch_connector = QAction(MainWindow)
        self.actionSwitch_connector.setObjectName(u"actionSwitch_connector")
        self.actionSwitch_connector.setEnabled(False)
        self.centralwidget = QWidget(MainWindow)
        self.centralwidget.setObjectName(u"centralwidget")
        MainWindow.setCentralWidget(self.centralwidget)
        self.dockWidget_source_tables = QDockWidget(MainWindow)
        self.dockWidget_source_tables.setObjectName(u"dockWidget_source_tables")
        self.dockWidgetContents = QWidget()
        self.dockWidgetContents.setObjectName(u"dockWidgetContents")
        self.verticalLayout_2 = QVBoxLayout(self.dockWidgetContents)
        self.verticalLayout_2.setSpacing(3)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.verticalLayout_2.setContentsMargins(3, 3, 3, 3)
        self.source_list = MultiCheckableListView(self.dockWidgetContents)
        self.source_list.setObjectName(u"source_list")
        self.source_list.setContextMenuPolicy(Qt.CustomContextMenu)
        self.source_list.setSelectionMode(QAbstractItemView.ExtendedSelection)
        self.source_list.setTextElideMode(Qt.ElideLeft)

        self.verticalLayout_2.addWidget(self.source_list)

        self.dockWidget_source_tables.setWidget(self.dockWidgetContents)
        MainWindow.addDockWidget(Qt.LeftDockWidgetArea, self.dockWidget_source_tables)
        self.dockWidget_source_options = QDockWidget(MainWindow)
        self.dockWidget_source_options.setObjectName(u"dockWidget_source_options")
        self.source_options_contents = QWidget()
        self.source_options_contents.setObjectName(u"source_options_contents")
        self.verticalLayout_5 = QVBoxLayout(self.source_options_contents)
        self.verticalLayout_5.setSpacing(3)
        self.verticalLayout_5.setObjectName(u"verticalLayout_5")
        self.verticalLayout_5.setContentsMargins(3, 3, 3, 3)
        self.dockWidget_source_options.setWidget(self.source_options_contents)
        MainWindow.addDockWidget(Qt.LeftDockWidgetArea, self.dockWidget_source_options)
        self.dockWidget_source_data = QDockWidget(MainWindow)
        self.dockWidget_source_data.setObjectName(u"dockWidget_source_data")
        self.dockWidgetContents_3 = QWidget()
        self.dockWidgetContents_3.setObjectName(u"dockWidgetContents_3")
        self.verticalLayout_3 = QVBoxLayout(self.dockWidgetContents_3)
        self.verticalLayout_3.setObjectName(u"verticalLayout_3")
        self.verticalLayout_3.setContentsMargins(6, 6, 6, 6)
        self.source_data_table = TableViewWithButtonHeader(self.dockWidgetContents_3)
        self.source_data_table.setObjectName(u"source_data_table")
        self.source_data_table.setContextMenuPolicy(Qt.CustomContextMenu)

        self.verticalLayout_3.addWidget(self.source_data_table)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.label = QLabel(self.dockWidgetContents_3)
        self.label.setObjectName(u"label")

        self.horizontalLayout.addWidget(self.label)

        self.default_column_type_combo_box = QComboBox(self.dockWidgetContents_3)
        self.default_column_type_combo_box.setObjectName(u"default_column_type_combo_box")

        self.horizontalLayout.addWidget(self.default_column_type_combo_box)

        self.horizontalSpacer = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.horizontalLayout.addItem(self.horizontalSpacer)


        self.verticalLayout_3.addLayout(self.horizontalLayout)

        self.dockWidget_source_data.setWidget(self.dockWidgetContents_3)
        MainWindow.addDockWidget(Qt.LeftDockWidgetArea, self.dockWidget_source_data)
        self.dockWidget_mappings = QDockWidget(MainWindow)
        self.dockWidget_mappings.setObjectName(u"dockWidget_mappings")
        self.dockWidgetContents_4 = QWidget()
        self.dockWidgetContents_4.setObjectName(u"dockWidgetContents_4")
        self.verticalLayout_7 = QVBoxLayout(self.dockWidgetContents_4)
        self.verticalLayout_7.setSpacing(3)
        self.verticalLayout_7.setObjectName(u"verticalLayout_7")
        self.verticalLayout_7.setContentsMargins(3, 3, 3, 3)
        self.button_layout = QHBoxLayout()
        self.button_layout.setObjectName(u"button_layout")
        self.button_layout.setContentsMargins(0, 0, 0, 0)
        self.new_button = QPushButton(self.dockWidgetContents_4)
        self.new_button.setObjectName(u"new_button")

        self.button_layout.addWidget(self.new_button)

        self.remove_button = QPushButton(self.dockWidgetContents_4)
        self.remove_button.setObjectName(u"remove_button")

        self.button_layout.addWidget(self.remove_button)

        self.duplicate_button = QPushButton(self.dockWidgetContents_4)
        self.duplicate_button.setObjectName(u"duplicate_button")

        self.button_layout.addWidget(self.duplicate_button)


        self.verticalLayout_7.addLayout(self.button_layout)

        self.mapping_list = QListView(self.dockWidgetContents_4)
        self.mapping_list.setObjectName(u"mapping_list")
        self.mapping_list.setContextMenuPolicy(Qt.CustomContextMenu)
        self.mapping_list.setSelectionMode(QAbstractItemView.ExtendedSelection)

        self.verticalLayout_7.addWidget(self.mapping_list)

        self.dockWidget_mappings.setWidget(self.dockWidgetContents_4)
        MainWindow.addDockWidget(Qt.RightDockWidgetArea, self.dockWidget_mappings)
        self.dockWidget_mapping_options = QDockWidget(MainWindow)
        self.dockWidget_mapping_options.setObjectName(u"dockWidget_mapping_options")
        self.mapping_options_contents = QWidget()
        self.mapping_options_contents.setObjectName(u"mapping_options_contents")
        self.formLayout_2 = QFormLayout(self.mapping_options_contents)
        self.formLayout_2.setObjectName(u"formLayout_2")
        self.formLayout_2.setHorizontalSpacing(3)
        self.formLayout_2.setVerticalSpacing(6)
        self.formLayout_2.setContentsMargins(3, 3, 3, 3)
        self.class_type_label = QLabel(self.mapping_options_contents)
        self.class_type_label.setObjectName(u"class_type_label")

        self.formLayout_2.setWidget(0, QFormLayout.LabelRole, self.class_type_label)

        self.class_type_combo_box = QComboBox(self.mapping_options_contents)
        self.class_type_combo_box.addItem("")
        self.class_type_combo_box.addItem("")
        self.class_type_combo_box.addItem("")
        self.class_type_combo_box.addItem("")
        self.class_type_combo_box.addItem("")
        self.class_type_combo_box.addItem("")
        self.class_type_combo_box.addItem("")
        self.class_type_combo_box.addItem("")
        self.class_type_combo_box.addItem("")
        self.class_type_combo_box.addItem("")
        self.class_type_combo_box.addItem("")
        self.class_type_combo_box.setObjectName(u"class_type_combo_box")

        self.formLayout_2.setWidget(0, QFormLayout.FieldRole, self.class_type_combo_box)

        self.import_objects_check_box = QCheckBox(self.mapping_options_contents)
        self.import_objects_check_box.setObjectName(u"import_objects_check_box")

        self.formLayout_2.setWidget(2, QFormLayout.LabelRole, self.import_objects_check_box)

        self.parameter_type_label = QLabel(self.mapping_options_contents)
        self.parameter_type_label.setObjectName(u"parameter_type_label")

        self.formLayout_2.setWidget(3, QFormLayout.LabelRole, self.parameter_type_label)

        self.parameter_type_combo_box = QComboBox(self.mapping_options_contents)
        self.parameter_type_combo_box.addItem("")
        self.parameter_type_combo_box.addItem("")
        self.parameter_type_combo_box.addItem("")
        self.parameter_type_combo_box.setObjectName(u"parameter_type_combo_box")

        self.formLayout_2.setWidget(3, QFormLayout.FieldRole, self.parameter_type_combo_box)

        self.value_type_label = QLabel(self.mapping_options_contents)
        self.value_type_label.setObjectName(u"value_type_label")

        self.formLayout_2.setWidget(4, QFormLayout.LabelRole, self.value_type_label)

        self.value_type_combo_box = QComboBox(self.mapping_options_contents)
        self.value_type_combo_box.addItem("")
        self.value_type_combo_box.addItem("")
        self.value_type_combo_box.addItem("")
        self.value_type_combo_box.addItem("")
        self.value_type_combo_box.addItem("")
        self.value_type_combo_box.setObjectName(u"value_type_combo_box")

        self.formLayout_2.setWidget(4, QFormLayout.FieldRole, self.value_type_combo_box)

        self.before_alternative_check_box = QCheckBox(self.mapping_options_contents)
        self.before_alternative_check_box.setObjectName(u"before_alternative_check_box")

        self.formLayout_2.setWidget(5, QFormLayout.LabelRole, self.before_alternative_check_box)

        self.read_start_row_label = QLabel(self.mapping_options_contents)
        self.read_start_row_label.setObjectName(u"read_start_row_label")

        self.formLayout_2.setWidget(6, QFormLayout.LabelRole, self.read_start_row_label)

        self.start_read_row_spin_box = QSpinBox(self.mapping_options_contents)
        self.start_read_row_spin_box.setObjectName(u"start_read_row_spin_box")
        self.start_read_row_spin_box.setMinimum(1)

        self.formLayout_2.setWidget(6, QFormLayout.FieldRole, self.start_read_row_spin_box)

        self.ignore_columns_label = QLabel(self.mapping_options_contents)
        self.ignore_columns_label.setObjectName(u"ignore_columns_label")

        self.formLayout_2.setWidget(7, QFormLayout.LabelRole, self.ignore_columns_label)

        self.ignore_columns_button = QPushButton(self.mapping_options_contents)
        self.ignore_columns_button.setObjectName(u"ignore_columns_button")

        self.formLayout_2.setWidget(7, QFormLayout.FieldRole, self.ignore_columns_button)

        self.dimension_label = QLabel(self.mapping_options_contents)
        self.dimension_label.setObjectName(u"dimension_label")

        self.formLayout_2.setWidget(8, QFormLayout.LabelRole, self.dimension_label)

        self.dimension_spin_box = QSpinBox(self.mapping_options_contents)
        self.dimension_spin_box.setObjectName(u"dimension_spin_box")
        self.dimension_spin_box.setMinimum(1)

        self.formLayout_2.setWidget(8, QFormLayout.FieldRole, self.dimension_spin_box)

        self.time_series_repeat_check_box = QCheckBox(self.mapping_options_contents)
        self.time_series_repeat_check_box.setObjectName(u"time_series_repeat_check_box")

        self.formLayout_2.setWidget(9, QFormLayout.LabelRole, self.time_series_repeat_check_box)

        self.map_dimensions_label = QLabel(self.mapping_options_contents)
        self.map_dimensions_label.setObjectName(u"map_dimensions_label")

        self.formLayout_2.setWidget(10, QFormLayout.LabelRole, self.map_dimensions_label)

        self.map_dimension_spin_box = QSpinBox(self.mapping_options_contents)
        self.map_dimension_spin_box.setObjectName(u"map_dimension_spin_box")
        self.map_dimension_spin_box.setMinimum(1)

        self.formLayout_2.setWidget(10, QFormLayout.FieldRole, self.map_dimension_spin_box)

        self.map_compression_check_box = QCheckBox(self.mapping_options_contents)
        self.map_compression_check_box.setObjectName(u"map_compression_check_box")

        self.formLayout_2.setWidget(11, QFormLayout.LabelRole, self.map_compression_check_box)

        self.dockWidget_mapping_options.setWidget(self.mapping_options_contents)
        MainWindow.addDockWidget(Qt.RightDockWidgetArea, self.dockWidget_mapping_options)
        self.dockWidget_mapping_spec = QDockWidget(MainWindow)
        self.dockWidget_mapping_spec.setObjectName(u"dockWidget_mapping_spec")
        self.dockWidgetContents_6 = QWidget()
        self.dockWidgetContents_6.setObjectName(u"dockWidgetContents_6")
        self.verticalLayout_8 = QVBoxLayout(self.dockWidgetContents_6)
        self.verticalLayout_8.setSpacing(3)
        self.verticalLayout_8.setObjectName(u"verticalLayout_8")
        self.verticalLayout_8.setContentsMargins(3, 3, 3, 3)
        self.mapping_spec_table = QTableView(self.dockWidgetContents_6)
        self.mapping_spec_table.setObjectName(u"mapping_spec_table")
        self.mapping_spec_table.horizontalHeader().setStretchLastSection(True)

        self.verticalLayout_8.addWidget(self.mapping_spec_table)

        self.dockWidget_mapping_spec.setWidget(self.dockWidgetContents_6)
        MainWindow.addDockWidget(Qt.RightDockWidgetArea, self.dockWidget_mapping_spec)
        self.dockWidget_source_files = QDockWidget(MainWindow)
        self.dockWidget_source_files.setObjectName(u"dockWidget_source_files")
        self.dockWidgetContents_7 = QWidget()
        self.dockWidgetContents_7.setObjectName(u"dockWidgetContents_7")
        self.verticalLayout = QVBoxLayout(self.dockWidgetContents_7)
        self.verticalLayout.setSpacing(3)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.verticalLayout.setContentsMargins(3, 3, 3, 3)
        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.file_path_label = QLabel(self.dockWidgetContents_7)
        self.file_path_label.setObjectName(u"file_path_label")
        font = QFont()
        font.setPointSize(10)
        self.file_path_label.setFont(font)

        self.horizontalLayout_2.addWidget(self.file_path_label)

        self.comboBox_source_file = ElidedCombobox(self.dockWidgetContents_7)
        self.comboBox_source_file.setObjectName(u"comboBox_source_file")
        self.comboBox_source_file.setEnabled(False)
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.comboBox_source_file.sizePolicy().hasHeightForWidth())
        self.comboBox_source_file.setSizePolicy(sizePolicy)
        self.comboBox_source_file.setSizeAdjustPolicy(QComboBox.AdjustToMinimumContentsLengthWithIcon)
        self.comboBox_source_file.setMinimumContentsLength(0)
        self.comboBox_source_file.setDuplicatesEnabled(True)

        self.horizontalLayout_2.addWidget(self.comboBox_source_file)

        self.toolButton_browse_source_file = QToolButton(self.dockWidgetContents_7)
        self.toolButton_browse_source_file.setObjectName(u"toolButton_browse_source_file")
        icon = QIcon()
        icon.addFile(u":/icons/folder-open-solid.svg", QSize(), QIcon.Normal, QIcon.Off)
        self.toolButton_browse_source_file.setIcon(icon)

        self.horizontalLayout_2.addWidget(self.toolButton_browse_source_file)


        self.verticalLayout.addLayout(self.horizontalLayout_2)

        self.verticalSpacer = QSpacerItem(20, 78, QSizePolicy.Minimum, QSizePolicy.Expanding)

        self.verticalLayout.addItem(self.verticalSpacer)

        self.dockWidget_source_files.setWidget(self.dockWidgetContents_7)
        MainWindow.addDockWidget(Qt.LeftDockWidgetArea, self.dockWidget_source_files)
        QWidget.setTabOrder(self.new_button, self.remove_button)
        QWidget.setTabOrder(self.remove_button, self.mapping_list)
        QWidget.setTabOrder(self.mapping_list, self.class_type_combo_box)
        QWidget.setTabOrder(self.class_type_combo_box, self.value_type_combo_box)
        QWidget.setTabOrder(self.value_type_combo_box, self.start_read_row_spin_box)
        QWidget.setTabOrder(self.start_read_row_spin_box, self.ignore_columns_button)
        QWidget.setTabOrder(self.ignore_columns_button, self.dimension_spin_box)
        QWidget.setTabOrder(self.dimension_spin_box, self.map_dimension_spin_box)
        QWidget.setTabOrder(self.map_dimension_spin_box, self.mapping_spec_table)

        self.retranslateUi(MainWindow)

        QMetaObject.connectSlotsByName(MainWindow)
    # setupUi

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QCoreApplication.translate("MainWindow", u"Import Editor", None))
        self.export_mappings_action.setText(QCoreApplication.translate("MainWindow", u"Export mappings...", None))
        self.import_mappings_action.setText(QCoreApplication.translate("MainWindow", u"Import mappings...", None))
        self.actionSwitch_connector.setText(QCoreApplication.translate("MainWindow", u"Switch connector...", None))
        self.dockWidget_source_tables.setWindowTitle(QCoreApplication.translate("MainWindow", u"Source tables", None))
        self.dockWidget_source_options.setWindowTitle(QCoreApplication.translate("MainWindow", u"Source options", None))
        self.dockWidget_source_data.setWindowTitle(QCoreApplication.translate("MainWindow", u"Source data", None))
        self.label.setText(QCoreApplication.translate("MainWindow", u"Surplus column data type:", None))
#if QT_CONFIG(tooltip)
        self.default_column_type_combo_box.setToolTip(QCoreApplication.translate("MainWindow", u"Select data type for additional columns in variable-length pivoted source data.", None))
#endif // QT_CONFIG(tooltip)
        self.dockWidget_mappings.setWindowTitle(QCoreApplication.translate("MainWindow", u"Mappings", None))
        self.new_button.setText(QCoreApplication.translate("MainWindow", u"Add", None))
        self.remove_button.setText(QCoreApplication.translate("MainWindow", u"Remove", None))
        self.duplicate_button.setText(QCoreApplication.translate("MainWindow", u"Duplicate", None))
        self.dockWidget_mapping_options.setWindowTitle(QCoreApplication.translate("MainWindow", u"Mapping options", None))
        self.class_type_label.setText(QCoreApplication.translate("MainWindow", u"Item type:", None))
        self.class_type_combo_box.setItemText(0, QCoreApplication.translate("MainWindow", u"Object class", None))
        self.class_type_combo_box.setItemText(1, QCoreApplication.translate("MainWindow", u"Relationship class", None))
        self.class_type_combo_box.setItemText(2, QCoreApplication.translate("MainWindow", u"Object group", None))
        self.class_type_combo_box.setItemText(3, QCoreApplication.translate("MainWindow", u"Alternative", None))
        self.class_type_combo_box.setItemText(4, QCoreApplication.translate("MainWindow", u"Scenario", None))
        self.class_type_combo_box.setItemText(5, QCoreApplication.translate("MainWindow", u"Scenario alternative", None))
        self.class_type_combo_box.setItemText(6, QCoreApplication.translate("MainWindow", u"Parameter value list", None))
        self.class_type_combo_box.setItemText(7, QCoreApplication.translate("MainWindow", u"Feature", None))
        self.class_type_combo_box.setItemText(8, QCoreApplication.translate("MainWindow", u"Tool", None))
        self.class_type_combo_box.setItemText(9, QCoreApplication.translate("MainWindow", u"Tool feature", None))
        self.class_type_combo_box.setItemText(10, QCoreApplication.translate("MainWindow", u"Tool feature method", None))

        self.import_objects_check_box.setText(QCoreApplication.translate("MainWindow", u"Import objects", None))
        self.parameter_type_label.setText(QCoreApplication.translate("MainWindow", u"Parameter type:", None))
        self.parameter_type_combo_box.setItemText(0, QCoreApplication.translate("MainWindow", u"Value", None))
        self.parameter_type_combo_box.setItemText(1, QCoreApplication.translate("MainWindow", u"Definition", None))
        self.parameter_type_combo_box.setItemText(2, QCoreApplication.translate("MainWindow", u"None", None))

        self.value_type_label.setText(QCoreApplication.translate("MainWindow", u"Default value type:", None))
        self.value_type_combo_box.setItemText(0, QCoreApplication.translate("MainWindow", u"Single value", None))
        self.value_type_combo_box.setItemText(1, QCoreApplication.translate("MainWindow", u"Time series", None))
        self.value_type_combo_box.setItemText(2, QCoreApplication.translate("MainWindow", u"Time pattern", None))
        self.value_type_combo_box.setItemText(3, QCoreApplication.translate("MainWindow", u"Map", None))
        self.value_type_combo_box.setItemText(4, QCoreApplication.translate("MainWindow", u"Array", None))

#if QT_CONFIG(tooltip)
        self.before_alternative_check_box.setToolTip(QCoreApplication.translate("MainWindow", u"Enable or disable 'Before alternative name' mapping for scenario alternative item type.", None))
#endif // QT_CONFIG(tooltip)
        self.before_alternative_check_box.setText(QCoreApplication.translate("MainWindow", u"Use before alternative", None))
        self.read_start_row_label.setText(QCoreApplication.translate("MainWindow", u"Read data from row:", None))
        self.ignore_columns_label.setText(QCoreApplication.translate("MainWindow", u"Ignore columns:", None))
        self.ignore_columns_button.setText("")
        self.dimension_label.setText(QCoreApplication.translate("MainWindow", u"Number of dimensions:", None))
#if QT_CONFIG(tooltip)
        self.time_series_repeat_check_box.setToolTip(QCoreApplication.translate("MainWindow", u"Set the repeat flag for all imported time series", None))
#endif // QT_CONFIG(tooltip)
        self.time_series_repeat_check_box.setText(QCoreApplication.translate("MainWindow", u"Repeat time series", None))
        self.map_dimensions_label.setText(QCoreApplication.translate("MainWindow", u"Map dimensions:", None))
        self.map_compression_check_box.setText(QCoreApplication.translate("MainWindow", u"Compress Maps", None))
        self.dockWidget_mapping_spec.setWindowTitle(QCoreApplication.translate("MainWindow", u"Mapping specification", None))
        self.dockWidget_source_files.setWindowTitle(QCoreApplication.translate("MainWindow", u"Source files", None))
        self.file_path_label.setText(QCoreApplication.translate("MainWindow", u"File path:", None))
        self.toolButton_browse_source_file.setText(QCoreApplication.translate("MainWindow", u"...", None))
    # retranslateUi

